package com.zsc.edu.gateway.modules.iot.record.service.impl;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zsc.edu.gateway.modules.iot.record.entity.DataWarningVo;
import com.zsc.edu.gateway.modules.iot.record.entity.RecordData;
import com.zsc.edu.gateway.modules.iot.record.entity.RecordDataStatusVo;
import com.zsc.edu.gateway.modules.iot.record.repo.RecordDataRepository;
import com.zsc.edu.gateway.modules.iot.record.service.RecordDataService;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;

/**
 * @author zhuang
 */
@AllArgsConstructor
@Service
public class RecordDataServiceImpl extends ServiceImpl<RecordDataRepository, RecordData> implements RecordDataService {

    @Override
    public RecordData recordData(String clientId, JSONObject data) {
        RecordData recordData = new RecordData();
        recordData.setClientId(clientId);
        recordData.setContent(data);
        recordData.setRecordTime(LocalDateTime.now());
        baseMapper.insert(recordData);
        return recordData;
    }

    @Override
    public RecordDataStatusVo recordDataStatus() {
        RecordDataStatusVo recordDataStatusVo = new RecordDataStatusVo();
        recordDataStatusVo.setRecordCount(baseMapper.selectCount(null));
        recordDataStatusVo.setDataCount(baseMapper.selectCount(new LambdaQueryWrapper<RecordData>().isNotNull(RecordData::getContent)));
        return recordDataStatusVo;
    }

    @Override
    public DataWarningVo dataWarning() {
        DataWarningVo dataWarningVo = new DataWarningVo();
        LocalDateTime todayStart = LocalDateTime.now().withHour(0).withMinute(0).withSecond(0);
        long warningCount = baseMapper.countWarnings();
        long todayWarningCount = baseMapper.countTodayWarnings(todayStart);
        long firstWarningCount = baseMapper.countWarningsByBit(1);
        long secondWarningCount = baseMapper.countWarningsByBit(2);
        long thirdWarningCount = baseMapper.countWarningsByBit(4);
        dataWarningVo.setWarningCount(warningCount);
        dataWarningVo.setTodayWarningCount(todayWarningCount);
        dataWarningVo.setFirstWarningCount(firstWarningCount);
        dataWarningVo.setSecondWarningCount(secondWarningCount);
        dataWarningVo.setThirdWarningCount(thirdWarningCount);
        return dataWarningVo;
    }

}
